package jp.kwebs;

import java.util.List;

import javax.transaction.Transactional;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.eclipse.microprofile.rest.client.inject.RegisterRestClient;

@Path("/books")
@Produces(MediaType.APPLICATION_JSON)
@RegisterRestClient(configKey = "book-service")
public interface BookService {
	
	// idで検索する
	@GET
	@Path("/{id}")
	public Book getBookById(@PathParam("id") Long id);	
	
	// 全てのエンティティを得る
	@GET
	public List<Book> getAllBooks(); 
	
	// 検索式による検索、ソート、ページング
	@GET
	@Path("/query")
	public List<Book> getBooks (	
			@DefaultValue("")		@QueryParam("query") String query,
			@DefaultValue("price")	@QueryParam("sort") String sort,
			@DefaultValue("0")	  	@QueryParam("index") int index,
			@DefaultValue("3")		@QueryParam("size") int size );	

	// 検索結果の件数
	@GET
	@Path("/count")
	public long getCount(  @DefaultValue("") @QueryParam("query") String query);
	
	// 検索結果のページ数
	@GET
	@Path("/size")
	public int getSize( 
			@DefaultValue("")	@QueryParam("query") String query,	  // 検索式
			@DefaultValue("3")	@QueryParam("size") int size     );
	
	// 新規作成
	@POST
	@Transactional
	@Consumes(MediaType.APPLICATION_JSON)
	public Response create(Book book);
	
	// 更新
	@PUT
	@Transactional
	@Consumes(MediaType.APPLICATION_JSON)
	public Response update(Book book);
	
	// 削除
	@DELETE
	@Transactional
	@Path("/delete/{id}")
	public Response delete(@PathParam("id") Long id);
	
	
	
}
